#ifndef LIBKYNETINFO_H
#define LIBKYNETINFO_H

/**
 * @file libkynetinfo.h
 * @author liuyunhe (liuyunhe@kylinos.cn)
 * @brief 网络信息
 * @version 0.1
 * @date 2023-2-17
 * 
 * @copyright Copyright (c) 2021
 * @defgroup libkysdk-net
 * @{
 * 
 */

#define IN 
#define OUT

#ifdef __cplusplus
extern "C"
{
#endif

typedef struct route
{
    char name[32]; //网卡名
    char addr[16]; //网关地址
    struct route *next;
} routeMapList, *prouteMapList;

typedef struct chain_data
{
    char target[8]; 
    char prot[4 + 1];
    char opt[4 + 1];
    char source[32];
    char destination[64];
    char option[128];
    struct chain_data *next;
} chainData, *pChainData;

typedef struct chain
{
    char total[16]; //链名
    char policy[32]; //链的规则
    pChainData data;
    struct chain *next;
} Chain, *pChain;

/**
 * @brief 获取指定网络端口的状态
 * 
 * @param port 端口号
 * @return int 端口状态 TCP_ESTABLISHED:0x1   TCP_SYN_SENT:0x2    TCP_SYN_RECV:0x3    TCP_FIN_WAIT1:0x4    TCP_FIN_WAIT2:0x5
    TCP_TIME_WAIT:0x6    TCP_CLOSE:0x7    TCP_CLOSE_WAIT:0x8    TCP_LAST_ACL:0x9    TCP_LISTEN:0xa    TCP_CLOSING:0xb
 */
extern int kdk_net_get_port_stat(int port);

/**
 * @brief 获取区间端口状态
 * 
 * @param start 开始端口
 * @param end 结束端口
 * @param result 区间端口状态接收缓存区
 * @return int 成功返回0 失败返回-1
 */
extern int kdk_net_get_multiple_port_stat(int start, int end, int *result);

/**
 * @brief 获取网关信息
 *
 * @return prouteMapList 网关信息存储结构体
 */
extern prouteMapList kdk_net_get_route();

/**
 * @brief 释放由kdk_net_get_route返回的网关信息结构体
 * 
 * @param list kdk_net_get_route的返回值
 */
extern void kdk_net_free_route(prouteMapList list);

/**
 * @brief 获取防火墙信息 iptables -L返回
 * 
 * @return pChain 防火墙信息结构体
 */
extern pChain kdk_net_get_iptable_rules();

/**
 * @brief 释放由kdk_net_get_iptable_rules返回的防火墙信息结构体
 *
 * @param list kdk_net_get_iptable_rules的返回值
 */
extern void kdk_net_free_chain(pChain list);

/**
 * @brief 获取子网信息
 * 
 * @param nc 网卡名称，如eno1
 * @param mask 子网掩码
 * 
 */
extern void kdk_net_get_netmask(IN const char *nc, OUT char *mask);

/**
 * @brief 获取进程对应端口
 * 
 * 
 * @return char ** 进程对应端口, 以NULL表示结尾，由alloc生成，需要被kdk_net_freeall回收；若获取出错，返回NULL；
 */
extern char **kdk_net_get_proc_port();

/**
 * @brief 获取所有UP状态的端口号
 * 
 * 
 * @return char ** 所有UP状态的端口号, 以NULL表示结尾，由alloc生成，需要被kdk_net_freeall回收；若获取出错，返回NULL；
 */
extern char **kdk_net_get_up_port();

/**
 * @brief  获取hosts配置
 * 
 * @return char* 成功返回hosts配置，失败返回NULL。返回的字符串需要被 free 释放。
 */
extern char* kdk_net_get_hosts();

/**
 * @brief  获取hosts配置的域名映射
 * 
 * @return char* 成功返回hosts配置的域名映射，失败返回NULL。返回的字符串需要被 free 释放。
 */
extern char* kdk_net_get_hosts_domain();

/**
 * @brief  获取DNS配置文件
 * 
 * @return char** DNS配置文件, 以NULL表示结尾，由alloc生成，需要被kdk_net_freeall回收；若获取出错，返回NULL；
 */
extern char** kdk_net_get_resolv_conf();

/**
 * @brief 用于回收字符串列表
 * 
 * @param ptr 字符串列表
 */
extern inline void kdk_net_freeall(char **ptr);

#ifdef __cplusplus
}
#endif

#endif

/**
 * \example kysdk-system/src/net/test/kynet-test.c
 * 
 */
